
local MAJOR = "StarLibTooltipStats-1.0" 
local MINOR = 9
assert(LibStub, MAJOR.." requires LibStub") 
local LibTooltipStats = LibStub:NewLibrary(MAJOR, MINOR)
if not LibTooltipStats then return end

local pool = setmetatable({}, {__mode = "k"})

local objects = {}
local objectsDict = {}
local update
local frame = CreateFrame("Frame")
local tooltip = CreateFrame("GameTooltip", "StarLibTooltipStats")
local initialized

if not LibTooltipStats.__index then
	LibTooltipStats.__index = LibTooltipStats
end

function initialize()
	if not initialized then return end
	for i = 0, self.size do
		tooltip:AddDoubleLine(" ", " ")
		LibTooltipStats.leftLine[i] = _G["StarLIbTooltipStatsLeft" .. i]
		LibTooltipStats.rightLines[i] = _G["StarLibTooltipStatsRight" .. i]
	end
	initialized = true
end

function LibTooltipStats:New(size)
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end

	setmetatable(obj, self)
	
	tinsert(objects, obj)
	objectsDict[obj] = obj
	
	obj.size = size
	
	if not initialized then
		initialize()
	end
	
	return obj	
end

-- Delete a LibTooltipStats object
-- @name LibTooltipStats.Del
-- @usage LibTooltipStats:Del([this])
-- @param this A LibTooltipStats object
-- @return Nothing
function LibTooltipStats:Del(this)
	
	if this and type(this) == "table" then
		pool[this] = true
		for i, v in ipairs(objects) do
			if v == this then
				objectsDict[this] = nil
				tremove(objects, i)
			end
		end
	else
		pool[self] = true
		for i, v in ipairs(objects) do
			if v == this then
				objectsDict[this] = nil
				tremove(objects, i)
			end
		end
	end
end

local getLocation, getGuild, getName

-- Return the default unit tooltip's stats
-- @usage LibTooltipStats:GetValues()
-- @return Name, guild, and location
function LibTooltipStats:GetValues()
	tooltip:ClearLines()
	tooltip:SetUnit("mouseover")
	return getName(), getGuild(), getLocation()
end

local LEVEL_start = "^" .. (type(LEVEL) == "string" and LEVEL or "Level")
function getLocation()
    if UnitIsVisible("mouseover") or not UnitIsConnected("mouseover") then
        return nil
    end
    
    local left_2 = self.leftLines[2]:GetText()
    local left_3 = self.leftLines[3]:GetText()
    if not left_2 or not left_3 then
        return nil
    end
    local hasGuild = not left_2:find(LEVEL_start)
    local factionText = not hasGuild and left_3 or self.leftLines[4]:GetText()
    if factionText == PVP then
        factionText = nil
    end
    local hasFaction = factionText and not UnitPlayerControlled("mouseover") and not UnitIsPlayer("mouseover") and (UnitFactionGroup("mouseover") or factionList[factionText])
    if hasGuild and hasFaction then
        return self.leftLines[5]:GetText()
    elseif hasGuild or hasFaction then
        return self.leftLines[4]:GetText()
    else
        return left_3
    end
end

function getGuild()
    local left_2 = self.leftLines[2]:GetText()
    if left_2:find(LEVEL_start) then return nil end
    return "<" .. left_2 .. ">"
end

function getName()
	return self.leftLines[1]:GetText()
end